@@ -2,6 +2,12 @@
  *  Assigned to public domain.  Use as you wish without restriction.
  */
 
+// sigc::signal<>.slots() is deprecated, but let's keep the test if possible.
+// If libsigc++ is configured with -Dbuild-deprecated-api=false
+// (--disable-deprecated-api), SIGCXX_DISABLE_DEPRECATED is defined in
+// sigc++config.h. An undef at the start of this file has no effect.
+#undef SIGCXX_DISABLE_DEPRECATED
+
 #include "testutilities.h"
 #include <sigc++/trackable.h>
 #include <sigc++/signal.h>
@@ -50,8 +56,8 @@ struct B : public sigc::trackable
 {
   B()
   {
-    sig.connect(sigc::mem_fun(this, &B::destroy));
-    sig.connect(sigc::mem_fun(this, &B::boom));
+    sig.connect(sigc::mem_fun(*this, &B::destroy));
+    sig.connect(sigc::mem_fun(*this, &B::boom));
     sig.connect(sigc::ptr_fun(&good_bye_world));
   }
 
@@ -89,7 +95,7 @@ int main(int argc, char* argv[])
 
   {
     A a;
-    sig.connect(sigc::mem_fun1(&a, &A::foo));
+    sig.connect(sigc::mem_fun1(a, &A::foo));
     confoo = sig.connect(sigc::ptr_fun1(&foo));
     conbar = sig.connect(sigc::ptr_fun1(&bar));
     result_stream << "sig is connected to A::foo, foo, bar (size=" << sig.size() << "): ";
@@ -103,11 +109,20 @@ int main(int argc, char* argv[])
   util->check_result(result_stream, "sig is connected to foo, bar (size=2): foo(2) bar(2) ");
 
   A a;                  // iterators stay valid after further connections.
-  cona = sig.slots().insert(conbar, sigc::mem_fun1(&a, &A::foo));
+#ifndef SIGCXX_DISABLE_DEPRECATED
+  cona = sig.slots().insert(conbar, sigc::mem_fun1(a, &A::foo));
+#else
+  cona = sig.connect(sigc::mem_fun1(a, &A::foo));
+#endif
   result_stream << "sig is connected to foo, A::foo, bar (size=" << sig.size() << "): ";
   sig(3);
+#ifndef SIGCXX_DISABLE_DEPRECATED
   util->check_result(result_stream,
     "sig is connected to foo, A::foo, bar (size=3): foo(3) A::foo(3) bar(3) ");
+#else
+  util->check_result(result_stream,
+    "sig is connected to foo, A::foo, bar (size=3): foo(3) bar(3) A::foo(3) ");
+#endif
 
   conbar->disconnect(); // manual disconnection
   result_stream << "sig is connected to foo, A::foo (size=" << sig.size() << "): ";
@@ -128,7 +143,7 @@ int main(int argc, char* argv[])
 
   {
     A a2;
-    sig.connect(sigc::compose(sigc::mem_fun(&a2, &A::foo), &foo));
+    sig.connect(sigc::compose(sigc::mem_fun(a2, &A::foo), &foo));
     result_stream << "sig is connected to compose(A::foo, foo) (size=" << sig.size() << "): ";
     sig(7);
     util->check_result(result_stream, "sig is connected to compose(A::foo, foo) (size=1): foo(7) A::foo(1) ");
@@ -139,7 +154,7 @@ int main(int argc, char* argv[])
 
   { // A slot# within a slot
     A a2;
-    sigc::slot1<int, int> setter = sigc::mem_fun(&a2, &A::foo);
+    sigc::slot1<int, int> setter = sigc::mem_fun(a2, &A::foo);
     sig.connect(sigc::compose(setter, &foo));
     result_stream << "sig is connected to compose(slot1(A::foo), foo) (size=" << sig.size() << "): ";
     sig(9);
@@ -151,7 +166,7 @@ int main(int argc, char* argv[])
 
   { // A slot within a slot
     A a2;
-    sigc::slot<int, int> setter = sigc::mem_fun(&a2, &A::foo);
+    sigc::slot<int, int> setter = sigc::mem_fun(a2, &A::foo);
     sig.connect(sigc::compose(setter, &foo));
     result_stream << "sig is connected to compose(slot(A::foo), foo) (size=" << sig.size() << "): ";
     sig(11);
